<?php

namespace Dropp\Woocommerce;

class Plugin {

	/**
	 * The variable that holds the current plugin version.
	 *
	 * @var string
	 */
	protected static $plugin_version = '1.2.0';

	/**
	 * The variable name is used as the text domain when internationalizing strings
	 * of text. Its value should match the Text Domain file header in the main
	 * plugin file.
	 *
	 * @var      string
	 */
	protected static $plugin_slug = 'woocommerce-with-dropp';

	/**
	 * Instance of this class.
	 *
	 * @var      object
	 */
	protected static $instance = null;

	/**
	 * Setup instance attributes
	 */
	private function __construct() {
		add_filter( 'woocommerce_shipping_methods', array( $this, 'include_shipping_method' ) );
		add_action( 'rest_api_init', [ $this, 'register_rest_routes' ] );
		add_filter( 'woocommerce_cart_shipping_method_full_label', [
			$this,
			'filter_woocommerce_cart_shipping_method_full_label'
		], 10, 2 );

		if ( ! is_admin() ) {
			add_action( 'init', [ $this, 'enqueue_public_assets' ] );
		}

		$this->register_payment_hooks();
	}

	/**
	 * Adds the logo to the shipping method label
	 *
	 * @param $label
	 * @param $method
	 *
	 * @return mixed|string
	 */
	function filter_woocommerce_cart_shipping_method_full_label( $label, $method ) {
		if ( $method->method_id == "with_dropp" ) {
			$label = $label . '<img src="' . WC_DROPP_URL . '/assets/images/logo.png' . '" style="width: 60px; height: auto; margin: 2px;">';
		}

		return $label;
	}

	/**
	 * Return the plugin slug.
	 *
	 * @return   string Plugin slug variable.
	 */
	public function get_plugin_slug() {
		return self::$plugin_slug;
	}

	/**
	 * Return the plugin version.
	 *
	 * @return   string Plugin slug variable.
	 */
	public function get_plugin_version() {
		return self::$plugin_version;
	}

	/**
	 * Fired when the plugin is deactivated.
	 */
	public static function deactivate() {
		deactivate_plugins( self::$plugin_slug );
	}

	/**
	 * Enqueues all the public assets needed to show the date picker component.
	 */
	public function enqueue_public_assets() {
		$assets = new Assets();
		add_action( 'woocommerce_review_order_before_payment', [ $assets, 'add_placeholder_div' ] );
		$assets->enqueue_scripts();
	}

	/**
	 * Includes the routes and registers them.
	 */
	function register_rest_routes() {
		$rest = new Rest();
		$rest->register_routes();
	}

	/**
	 * Registers payment hooks to process the payment of an order.
	 */
	function register_payment_hooks() {
		$payment = new Payment();
		$payment->register_hooks();
	}

	/**
	 * Include hook to checkout page.
	 *
	 * @return  array<string>
	 */
	function include_shipping_method( array $methods ) {
		$methods['with_dropp'] = '\Dropp\Woocommerce\ShippingMethod';

		return $methods;
	}

	/**
	 * Return an instance of this class.
	 *
	 * @return    Plugin    A single instance of this class.
	 */
	public static function get_instance() {
		// If the single instance hasn't been set, set it now.
		if ( null == self::$instance ) {
			self::$instance = new self;
		}

		return self::$instance;
	}
}
