<?php

namespace Dropp\Woocommerce;

use WP_Error;
use WP_REST_Request;
use WP_REST_Response;

class Rest {

	private $session;

	/**
	 * Rest constructor.
	 */
	public function __construct() {
		$this->session = new Session();
	}

	/**
	 * Registers all routes.
	 */
	public function register_routes() {
		register_rest_route( 'dropp/v1', '/shippingmethod', [
			'methods'  => 'GET',
			'callback' => [ $this, 'get_selected_shipping_method' ]
		] );

		register_rest_route( 'dropp/v1', '/timeslots', [
			'methods'  => 'GET',
			'callback' => [ $this, 'get_timeslots' ]
		] );

		register_rest_route( 'dropp/v1', '/timeslot', [
			'methods'  => 'POST',
			'callback' => [ $this, 'save_selected_timeslot' ]
		] );

		register_rest_route( 'dropp/v1', '/timeslot', [
			'methods'  => 'GET',
			'callback' => [ $this, 'get_selected_timeslot' ]
		] );

		register_rest_route( 'dropp/v1', '/order', [
			'methods'  => 'PATCH',
			'callback' => [ $this, 'update_order_status' ]
		] );

		register_rest_route( 'dropp/v1', '/theme', [
			'methods'  => 'GET',
			'callback' => [ $this, 'get_theme' ]
		] );
	}

	/**
	 * @param WP_REST_Request $request
	 *
	 * @return mixed
	 */
	public function get_selected_shipping_method( WP_REST_Request $request ) {
		wc_load_cart();

		return WC()->session->get( 'chosen_shipping_methods' )[0];
	}

	/**
	 * Persists the selected timeslot to the current session so it can be
	 * used at a later moment.
	 *
	 * @param WP_REST_Request $request
	 *
	 * @return array
	 */
	public function save_selected_timeslot( WP_REST_Request $request ) {
		$timeslot = json_decode( $request->get_body(), true );
		$this->session->set_selected_timeslot( $timeslot );

		return [ 'message' => 'ok' ];
	}

	/**
	 * Retrieves current selected timeslot.
	 *
	 * @param WP_REST_Request $request
	 *
	 * @return array|null
	 */
	public function get_selected_timeslot( WP_REST_Request $request ) {
		return $this->session->get_selected_timeslot();
	}

	/**
	 * Return the available slots.
	 *
	 * @param WP_REST_Request $request
	 *
	 * @return array
	 */
	public function get_timeslots( WP_REST_Request $request ) {
		wc_load_cart();

		$selected_shipping_method = WC()->session->get( 'chosen_shipping_methods' )[0];

		// if Dropp is not selected, we don't need to show timeslots
		if ( $selected_shipping_method !== 'with_dropp' ) {
			return [];
		}

		$client    = ApiClient::get_client();
		$timeslots = $client->get( 'timeslots' );

		return array_map(
			function ( $item ) {
				return [
					'start'           => $item->slotStart,
					'durationMinutes' => $item->slotDurationMinutes
				];
			},
			json_decode( $timeslots->getBody() )
		);
	}

	public function update_order_status( WP_REST_Request $request ) {
		$data     = $request->get_json_params();
		$order_id = $data['externalId'];
		$order    = wc_get_order( $order_id );

		if ( ! $order ) {
			return new WP_Error( 'invalid_param', 'Order not found for id', [ 'status' => 404 ] );
		}

		$status = $data['status'];

		if ( ! $status ) {
			return new WP_Error( 'invalid_body', 'Invalid body request', [ 'status' => 400 ] );
		}

		if ( $status === 'DELIVERED' && $order->get_status() === 'processing' ) {
			$order->update_status( 'completed' );
		}

		$order->update_meta_data( 'dropp_order_status', $data['status'] );
		$order->save();

		return new WP_REST_Response( null, 204 );
	}

	public function get_theme( WP_REST_Request $request ) {

		$client   = ApiClient::get_client();
		$response = $client->get( 'theme' );
		$theme    = $response->getBody() ?: [];

		return json_decode( $theme );
	}

}