<?php

namespace Dropp\Woocommerce;

use GuzzleHttp\Exception\GuzzleException;
use WC_Shipping_Method;

class ShippingMethod extends WC_Shipping_Method {

	const PROD_API_URL = 'https://public-api.dropp.rocks';
	const SANDBOX_API_URL = 'https://public-api.qa.dropp.rocks';

	/**
	 * @var string
	 */
	public $push_order_on_status;

	/**
	 * @var string
	 */
	public $api_url = self::SANDBOX_API_URL;

	/**
	 * Constructor for your shipping class
	 */
	public function __construct( $instance_id = 0 ) {
		$this->id                 = 'with_dropp';
		$this->instance_id        = $instance_id;
		$this->method_title       = __( 'Dropp', 'woocommerce-with-dropp' );
		$this->method_description = __( 'Dropp is a fast, sustainable, and flexible 3-hour delivery for eCommerce shops', 'woocommerce-with-dropp' );
		$this->supports           = [ 'settings' ];

		$this->init();
	}

	/**
	 * Init your settings.
	 */
	function init() {
		// Load the settings.
		$this->init_form_fields();
		$this->init_settings();

		// Define user set variables.
		$this->title                = $this->get_option( 'title' );
		$this->enabled              = $this->get_option( 'enabled' );
		$this->push_order_on_status = $this->get_option( 'push_order_on_status' );
		$this->api_url              = $this->get_option( 'sandbox' ) === 'no' ? self::PROD_API_URL : self::SANDBOX_API_URL;

		if ( $this->enabled !== 'yes' ) {
			return;
		}

		// Actions
		add_action( 'woocommerce_update_options_shipping_' . $this->id, [ $this, 'process_admin_options' ] );

		// Filters
		add_filter( 'woocommerce_package_rates', [ $this, 'sort_shipping_methods' ], 10, 2 );
	}

	/**
	 * Initialise Gateway Settings Form Fields
	 */
	public function init_form_fields() {

		$this->form_fields = [
			'enabled'              => [
				'title'   => __( 'Enable/disable', 'woocommerce-with-dropp' ),
				'type'    => 'checkbox',
				'label'   => __( 'Enable Dropp shipping method', 'woocommerce-with-dropp' ),
				'default' => 'yes',
			],
			'title'                => [
				'title'       => __( 'Title', 'woocommerce-with-dropp' ),
				'type'        => 'text',
				'description' => __( 'This controls the title which the user sees during checkout.', 'woocommerce-with-dropp' ),
				'default'     => __( 'Dropp Shipping', 'woocommerce-with-dropp' ),
				'desc_tip'    => true,
			],
			'api_key'              => [
				'title'       => __( 'API key', 'woocommerce-with-dropp' ),
				'type'        => 'text',
				'description' => __( 'Place here the API Key received from Dropp', 'woocommerce-with-dropp' ),
				'desc_tip'    => true,
			],
			'shop_id'              => [
				'title'       => __( 'Shop ID', 'woocommerce-with-dropp' ),
				'type'        => 'text',
				'description' => __( 'Place here the Shop ID received from Dropp', 'woocommerce-with-dropp' ),
				'desc_tip'    => true,
			],
			'sandbox'              => [
				'title'   => __( 'Use Sandbox', 'woocommerce-with-dropp' ),
				'type'    => 'checkbox',
				'label'   => __( 'Use Dropp sandbox environment', 'woocommerce-with-dropp' ),
				'default' => 'yes',
			],
			'sort_rates_by_price'  => [
				'title'   => __( 'Sort rates by cost', 'woocommerce-with-dropp' ),
				'type'    => 'checkbox',
				'label'   => __( 'Sort the shipping rates from the cheapest to the most expensive', 'woocommerce-with-dropp' ),
				'default' => 'no',
			],
			'push_order_on_status' => [
				'title'   => __( 'Push order on status', 'woocommerce-with-dropp' ),
				'type'    => 'select',
				'label'   => __( 'Choose the status of the order to push to Dropp', 'woocommerce-with-dropp' ),
				'default' => 'wc-processing',
				'options' => wc_get_order_statuses(),
			],
		];

	}

	/**
	 * Get setting form fields for instances of this shipping method within zones.
	 *
	 * @return array
	 */
	public function get_instance_form_fields() {
		return parent::get_instance_form_fields();
	}

	/**
	 * calculate shipping cost.
	 *
	 * @since 1.0
	 **/
	public function calculate_shipping( $package = [] ) {
		$client = ApiClient::get_client( $this );
//		$shop_id = $this->get_option( 'shop_id' );
		$zipcode = $this->extract_zipcode( $package );

		// we don't proceed while there's no zipcode set in the cart
		if ( ! $zipcode ) {
			return;
		}

		try {
			$response = $client->get(
				'rates',
				[
					'query' => [
						'zip'         => $this->extract_zipcode( $package ),
						'sku'         => implode( ',', $this->extract_skus( $package ) ),
						'totalAmount' => $this->extract_total_amount( $package ),
						'totalCount'  => $this->extract_total_count( $package ),
						'currency'    => get_woocommerce_currency(),
					]
				]
			);

			if ( $response->getStatusCode() !== 200 ) {
				return;
			}

			$body = json_decode( $response->getBody() );

			// if the body is not a valid shipping rate object, ignore
			if ( ! $body ) {
				return;
			}

			// Register the rate
			$this->add_rate( [
				'label'    => $this->title,
				'cost'     => $body->amount,
				'calc_tax' => 'per_item',
			] );
		} catch ( GuzzleException $e ) {
			// do nothing for now but don't add any rate
		}
	}

	/**
	 * Sort the woocommerce rates from the cheapest to the most expensive
	 * in case the setting is enabled.
	 *
	 * @param $rates
	 * @param $package
	 *
	 * @return array
	 */
	public function sort_shipping_methods( $rates, $package ) {
		if ( $this->get_option( 'sort_rates_by_price' ) !== 'yes' ) {
			return $rates;
		}

		if ( empty( $rates || ! is_array( $rates ) ) ) {
			return $rates;
		}

		uasort( $rates, function ( $a, $b ) {
			if ( $a == $b ) {
				return 0;
			}

			return ( $a->cost < $b->cost ) ? - 1 : 1;
		} );

		return $rates;
	}

	/**
	 * Returns the zipcode or postcode assigned to the current cart. If it's not set,
	 * returns false.
	 *
	 * @param array $package
	 *
	 * @return string|false
	 */
	private function extract_zipcode( array $package ) {
		return $package &&
		       $package['destination'] &&
		       $package['destination']['postcode'] ?
			$package['destination']['postcode'] :
			false;
	}

	/**
	 * Returns the total cost of items in the package.
	 *
	 * @param array $package
	 *
	 * @return float
	 */
	private function extract_total_amount( array $package ) {
		return $package['contents_cost'];
	}

	/**
	 * Returns the total number of items in the package.
	 *
	 * @param array $package
	 *
	 * @return int
	 */
	private function extract_total_count( array $package ) {
		return array_reduce(
			array_map(
				function ( $item ) {
					return $item['quantity'];
				},
				array_values( $package['contents'] )
			),
			function ( $acc, $qty ) {
				return $acc + $qty;
			},
			0
		);
	}

	/**
	 * Maps products inside the package to a list of product SKUs.
	 *
	 * @return array<string>
	 */
	private function extract_skus( $package ) {
		$skus = [];

		foreach ( $package['contents'] as $values ) {
			$skus[] = $values['data']->get_sku();
		}

		return $skus;
	}
}
